import React, { Component, Fragment } from 'react';
import DropZone from './drop-zone';
import ResizeImage from '../lib/resize-image/util';
import Cropper from 'cropperjs';
import classnames from 'classnames';
import 'cropperjs/dist/cropper.css';
import 'rc-slider/assets/index.css';
import { hasClass } from '../lib/dom';
import { ExtendedFile } from '../types/media';
import { getScrollTop } from '../lib/utility';
import redoIcon from '../assets/images/media-redo.svg';
import reflectVerticalIcon from '../assets/images/media-reflect-vertical.svg';
import reflectHorizontalIcon from '../assets/images/media-reflect-horizontal.svg';
import rotateIcon from '../assets/images/media-rotate.svg';
import cropCustomIcon from '../assets/images/media-crop-custom.svg';
import crop11Icon from '../assets/images/media-crop-1_1.svg';
import crop43Icon from '../assets/images/media-crop-4_3.svg';
import crop169Icon from '../assets/images/media-crop-16_9.svg';

const FreeStyle = require('free-style');

const Style = FreeStyle.create();
const mediaEditStyle: string = Style.registerStyle({
  display: 'block',
  '.acms-admin-modal-body': {
    backgroundColor: '#464E5A',
    padding: '40px 0 10px 0'
  },
  '.acms-admin-modal-dialog': {
    maxWidth: '1000px'
  },
  '.acms-admin-cropper-wrap': {
    'max-width': '756px',
    img: {
      width: '100%',
      height: 'auto'
    }
  },
  '.cropper-bg': {
    'background-color': '#FFFFFF',
    'background-image': 'url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAGXRFWHRTb2Z0d2FyZQBBZG9iZSBJbWFnZVJlYWR5ccllPAAAAC1JREFUeNpi5GBhkWLAA378+fMMnzwTA4Vg1IDBYAAjIQWE0sloIA4LAwACDABOxARJOkvQxAAAAABJRU5ErkJggg==)'
  },
  '.acms-admin-cropper-container': {
    display: 'flex',
    width: '100%',
    flexWrap: 'noWrap'
  },
  '.acms-admin-cropper-wrap': {
    flex: 1,
    maxWidth: '750px',
    boxSizing: 'border-box',
    img: {
      maxWidth: '100%'
    }
  },
  '.acms-admin-cropper-target': {
    width: '100%'
  },
  '.acms-admin-cropper-tools': {
    width: '130px',
    padding: '0 15px',
    boxSizing: 'border-box'
  },
  '.acms-admin-cropper-input-wrap': {
    width: '130px',
    padding: '0 15px',
    boxSizing: 'border-box',
    input: {
      maxWidth: '84px'
    }
  },
  '.acms-admin-cropper-input': {
    backgroundColor: '#2F3641',
    color: '#FFF',
    borderRadius: '3px',
    fontSize: '14px',
    padding: '7px 10px',
    border: 'none',
    'margin-bottom': '5px'
  },
  '.acms-admin-cropper-input-label': {
    fontSize: '10px',
    display: 'inline-block',
    color: '#FFF'
  },
  '.acms-admin-cropper-btn': {
    width: '60px',
    backgroundColor: 'transparent',
    appearance: 'none',
    border: 'none',
    color: '#FFF',
    display: 'block',
    margin: '0 auto 30px auto',
    fontSize: '10px',
    padding: '0',
    transition: 'background .3s',
    '&:hover': {
      backgroundColor: '#333A45',
      borderRadius: '3px'
    }
  },
  '.acms-admin-cropper-icon': {
    width: '30px',
    height: 'auto',
    display: 'block',
    margin: '0 auto 5px auto'
  },
  '.acms-admin-cropper-icon-crop': {
    width: '40px',
    height: 'auto',
    display: 'block',
    margin: '0 auto 5px auto'
  },
  '.acms-admin-cropper-reflect-group': {
    display: 'table',
    width: '100%',
    borderRadius: '3px',
    border: '1px solid #333A45'
  },
  '.acms-admin-cropper-reflect-btn': {
    backgroundColor: 'transparent',
    width: '50%',
    display: 'table-cell',
    border: 'none',
    color: '#FFF',
    padding: '10px 15px'
  },
  '.acms-admin-cropper-reflect-icon': {
    width: '20px',
    height: 'auto',
    display: 'block',
    margin: '0 auto 5px auto',
  },
  '.acms-admin-cropper-reflect-label': {
    textAlign: 'center',
    color: '#FFF',
    fontSize: '10px',
    marginBottom: '30px'
  },
  '.acms-admin-cropper-crop-group-wrap': {
    flex: '1'
  },
  '.acms-admin-cropper-crop-group': {
    display: 'table'
  },
  '.acms-admin-cropper-crop-label': {
    display: 'table-cell',
    verticalAlign: 'middle',
    color: '#FFF',
    fontSize: '12px',
    paddingRight: '20px'
  },
  '.acms-admin-cropper-crop-btn': {
    borderRadius: '3px',
    backgroundColor: 'transparent',
    color: '#FFF',
    display: 'table-cell',
    fontSize: '12px',
    border: 'none',
    padding: '10px 12px',
    verticalAlign: 'middle',
    transition: 'background .3s',
    '&.active': {
      backgroundColor: '#333A45',
      borderRadius: '3px'
    },
    '&:hover': {
      backgroundColor: '#333A45',
      borderRadius: '3px'
    }
  },
  '.acms-admin-cropper-crop-btn-block': {
    display: 'block'
  },
  '.acms-admin-range-slider': {
    width: '200px'
  },
  '.acms-admin-cropper-img-container': {
    width: '100%',
    height: '490px'
  },
  '.acms-admin-cropper-focal-point': {
    position: 'absolute',
    width: '12px',
    height: '12px',
    borderRadius: '50%',
    border: '5px solid #FFF',
    left: '50%',
    top: '50%',
    marginTop: '-6px',
    marginLeft: '-6px'
  },
  '.acms-admin-cropper-bottom': {
    display: 'flex',
    width: '100%',
    alignItems: 'center'
  },
  '.cropper-point': {
    width: '10px',
    height: '10px',
    backgroundColor: '#FFF',
    borderRadius: '50%',
    '&.point-n': {
      marginLeft: '-5px',
      top: '-5px'
    },
    '&.point-e': {
      marginTop: '-5px',
      right: '-5px'
    },
    '&.point-w': {
      left: '-5px',
      marginTop: '-5px'
    },
    '&.point-s': {
      bottom: '-5px',
      marginLeft: '-5px'
    },
    '&.point-ne': {
      right: '-5px',
      top: '-5px'
    },
    '&.point-nw': {
      left: '-5px',
      top: '-5px'
    },
    '&.point-sw': {
      bottom: '-5px',
      left: '-5px'
    },
    '&.point-se': {
      bottom: '-5px',
      right: '-5px'
    }
  },
  '.cropper-line': {
    backgroundColor: '#FFF'
  },
  '.cropper-view-box': {
    outline: '1px solid #FFF',
    outlineColor: '#FFF'
  },
  '.rc-slider-step': {
    backgroundColor: '#e9e9e9'
  },
  '.rc-slider-handle': {
    backgroundColor: '#FFF',
    width: '17px',
    height: '17px',
    border: 'none',
    marginLeft: '-6px',
    marginTop: '-6px'
  },
  '.acms-admin-range-label': {
    fontSize: '12px',
    color: '#FFF'
  },
  '.acms-admin-range-shape-wrap': {
    padding: '0 15px',
    color: '#FFF',
    display: 'inline-block',
    fontSize: '10px',
    verticalAlign: 'middle',
    textAlign: 'center',
    position: 'relative'
  },
  '.acms-admin-range-shape-small': {
    width: '16px',
    height: '16px',
    display: 'block',
    backgroundColor: '#FFF',
    borderRadius: '3px',
  },
  '.acms-admin-range-shape-large': {
    width: '30px',
    height: '30px',
    display: 'block',
    backgroundColor: '#FFF',
    borderRadius: '3px',
  },
  '.acms-admin-range-shape-text': {
    position: 'absolute',
    bottom: '-20px',
    left: '50%',
    marginLeft: '-5px'
  }
});

const styleElement = Style.getStyles();
const resizeImage = new ResizeImage();

interface MediaEditModalProp {
  onClose: Function,
  src: string,
  focalPoint: string
}

interface MediaEditModalState {
  blob: Blob | null,
  upload: boolean,
  displayWidth: number | null,
  displayHeight: number | null,
  useFocalPoint: boolean,
  focalPoint: [number, number],
  zoom: number,
  ratio: number,
  src: string
}

export default class MediaEditModal extends Component<MediaEditModalProp, MediaEditModalState> {

  img: HTMLImageElement;
  cropper: Cropper;
  container: HTMLDivElement;

  constructor(props) {
    super(props);
    let focalPoint = [50, 50];
    let useFocalPoint = false;
    if (props.focalPoint) {
      focalPoint = props.focalPoint.split(',');
      focalPoint = focalPoint.map((str) => parseFloat(str));
      useFocalPoint = true;
    }
    this.state = {
      blob: null,
      upload: false,
      displayWidth: 0,
      displayHeight: 0,
      zoom: 1,
      ratio: 0,
      src: '',
      useFocalPoint,
      focalPoint
    };
  }

  onComplete(files: ExtendedFile[]) {
    resizeImage.getUrlFromFile(files[0], false, 400, false).then((src) => {
      this.setState({ src });
    });
  }

  onCancel() {
    this.props.onClose();
  }

  onCrop() {
    let mimeType = 'image/png';
    if (this.img.src.slice(-3) !== 'png') {
      mimeType = 'image/jpeg';
    }
    this.cropper.getCroppedCanvas().toBlob((blob) => {
      const { focalPoint, useFocalPoint } = this.state;
      if (useFocalPoint) {
        this.props.onClose(blob, focalPoint);
      } else {
        this.props.onClose(blob, null);
      }
    }, mimeType);
  }

  reset() {
    this.cropper.reset();
  }

  componentDidMount() {
    this.setDisplaySize();
    this.cropper = new Cropper(this.img, {
      autoCrop: false,
      zoomable: false,
      responsive: true,
      crop: (e) => {
        const data = e.detail;
        if (data.width || data.height) {
          this.setState({
            displayWidth: data.width,
            displayHeight: data.height
          });
        } else {
          this.setState({
            displayWidth: this.img.width,
            displayHeight: this.img.height
          });
        }
      },
      autoCropArea: 1,
      dragMode: 'none'
    });
  }

  setDisplaySize() {
    if (this.img.complete) {
      this.setState({
        displayWidth: this.img.width,
        displayHeight: this.img.height
      });
    }
    this.img.onload = () => {
      this.setState({
        displayWidth: this.img.width,
        displayHeight: this.img.height
      });
    }
  }

  onUploadImg(files) {
    this.cropper.replace(files[0].preview);
  }

  useOriginalImg() {
    this.cropper.replace(this.props.original);
  }

  changeAspectRatio(ratioProp) {
    const { ratio } = this.state;
    this.cropper.clear();
    if (ratioProp !== ratio) {
      this.cropper.setAspectRatio(ratioProp);
      this.cropper.crop();
      this.setState({
        ratio: ratioProp
      });
    } else {
      this.setState({
        ratio: 0
      });
    }
  }

  flipX() {
    this.cropper.scaleX(-this.cropper.getData().scaleX || -1);
  }

  flipY() {
    this.cropper.scaleY(-this.cropper.getData().scaleY || -1);
  }

  rotate(rotate) {
    this.cropper.rotate(rotate);
  }

  zoom(zoom) {
    const defaultZoomRatio = this.cropper.canvasData.naturalWidth / this.cropper.containerData.width;
    this.setState({ zoom }, () => {
      this.cropper.zoomTo(zoom / defaultZoomRatio);
    });
  }

  toggleFocalPoint() {
    const { useFocalPoint } = this.state;
    this.setState({
      useFocalPoint: !useFocalPoint
    });
  }

  changeFocalPoint(e) {
    const { useFocalPoint } = this.state;
    if (!useFocalPoint) {
      return;
    }
    if (e.target !== this.container && !hasClass(e.target, 'cropper-drag-box')) {
      return;
    }
    const rect = e.target.getBoundingClientRect();
    const targetX = rect.left;
    const targetY = rect.top + getScrollTop();
    const posX = e.pageX - targetX;
    const posY = e.pageY - targetY;
    const percentX = posX * 100 / this.container.offsetWidth;
    const percentY = posY * 100 / this.container.offsetHeight;
    this.setState({
      focalPoint: [percentX, percentY]
    });
  }

  render() {
    const {
      displayWidth, displayHeight, zoom, ratio, useFocalPoint, focalPoint
    } = this.state;
    const { src, original } = this.props;
    return (
      <Fragment>
        <style>{styleElement}</style>
        <div className={classnames('acms-admin-modal', 'in', mediaEditStyle)} style={{ display: 'block', backgroundColor: 'rgba(0,0,0,.5)' }}>
          <div className="acms-admin-modal-dialog">
            <div className="acms-admin-modal-content">
              <div className="acms-admin-modal-header">
                <i className="acms-admin-modal-hide acms-admin-icon-delete" onClick={this.onCancel.bind(this)} />
                <h3>画像編集</h3>
              </div>
              <DropZone onComplete={this.onUploadImg.bind(this)}>
                <div className="acms-admin-modal-body">
                  <div className="acms-admin-cropper-container">
                    <div className="acms-admin-cropper-tools">
                      <button type="button" className="acms-admin-cropper-btn" onClick={this.reset.bind(this)}>
                        <img src={redoIcon} className="acms-admin-cropper-icon" />
                        元に戻す
                      </button>

                      <button type="button" className="acms-admin-cropper-btn" onClick={this.flipX.bind(this)}>
                        <img src={reflectHorizontalIcon} className="acms-admin-cropper-reflect-icon" style={{width: '32px'}}/>
                        反転（水平）
                      </button>
                      <button type="button" className="acms-admin-cropper-btn" onClick={this.flipY.bind(this)}>
                        <img src={reflectVerticalIcon} className="acms-admin-cropper-reflect-icon" style={{width: '32px'}} />
                        反転（垂直）
                      </button>

                      <p className="acms-admin-cropper-reflect-label">リフレクト</p>
                      <button type="button" className="acms-admin-cropper-btn" onClick={this.rotate.bind(this, -90)}>
                        <img src={rotateIcon} className="acms-admin-cropper-icon" />
                        回転
                      </button>
                    </div>
                    <div className="acms-admin-cropper-wrap">
                      <div className="acms-admin-cropper-img-container" style={{position: 'relative'}} ref={(container) => { this.container = container }} onClick={(e) => {
                        this.changeFocalPoint(e);
                      }}>
                        <img src={src} ref={(img: HTMLImageElement) => { this.img = img; }} className="acms-admin-cropper-target" />
                        {useFocalPoint && <div className="acms-admin-cropper-focal-point" style={{
                          left: `${focalPoint[0]}%`,
                          top: `${focalPoint[1]}%`
                        }}></div>}
                      </div>
                      <div className="acms-admin-form acms-admin-cropper-bottom" style={{ marginTop: '10px' }}>
                        <div className="acms-admin-cropper-crop-group-wrap">
                          <div className="acms-admin-cropper-crop-group">
                            <span className="acms-admin-cropper-crop-label">切り抜き</span>
                            <button
                              type="button"
                              className={classnames('acms-admin-cropper-crop-btn', {
                                active: ratio === null
                              })}
                              onClick={this.changeAspectRatio.bind(this, null)}
                            >
                              <img src={cropCustomIcon} className="acms-admin-cropper-icon-crop" />
                              カスタム
                            </button>
                            <button
                              type="button"
                              className={classnames('acms-admin-cropper-crop-btn', {
                                active: ratio === 1
                              })}
                              onClick={this.changeAspectRatio.bind(this, 1)}
                            >
                              <img src={crop11Icon} className="acms-admin-cropper-icon-crop" />
                              1:1
                            </button>
                            <button
                              type="button"
                              className={classnames('acms-admin-cropper-crop-btn', {
                                active: ratio === (4 / 3)
                              })}
                              onClick={this.changeAspectRatio.bind(this, 4 / 3)}
                            >
                              <img src={crop43Icon} className="acms-admin-cropper-icon-crop" />
                              4:3
                            </button>
                            <button
                              type="button"
                              className={classnames('acms-admin-cropper-crop-btn', {
                                active: ratio === (16 / 9)
                              })}
                              onClick={this.changeAspectRatio.bind(this, 16 / 9)}
                            >
                              <img src={crop169Icon} className="acms-admin-cropper-icon-crop" />
                              16:9
                            </button>
                          </div>
                        </div>
                      </div>
                    </div>
                    <div className="acms-admin-cropper-input-wrap">
                      <label className="acms-admin-cropper-input-label" htmlFor="cropper-width">幅</label>
                      <div>
                        <input type="text" value={Math.round(displayWidth)} readOnly className="acms-admin-cropper-input" id="cropper-width" />
                      </div>
                      <label className="acms-admin-cropper-input-label" htmlFor="cropper-height">高さ</label>
                      <div>
                        <input type="text" value={Math.round(displayHeight)} readOnly className="acms-admin-cropper-input" id="cropper-height" />
                      </div>
                      <button type="button" className={classnames('acms-admin-cropper-crop-btn acms-admin-cropper-crop-btn-block', {
                          active: useFocalPoint
                        })}
                        onClick={() => {
                          this.toggleFocalPoint();
                        }}
                      >
                      中心点
                      </button>
                      {original &&
                        <button className="acms-admin-cropper-crop-btn acms-admin-cropper-crop-btn-block" onClick={this.useOriginalImg.bind(this)}>元画像を使用</button>}
                    </div>
                  </div>
                </div>
              </DropZone>
              <div className="acms-admin-modal-footer">
                <button type="button" className="acms-admin-btn" onClick={this.onCancel.bind(this)}>キャンセル</button>
                <button type="button" className="acms-admin-btn acms-admin-btn-info" onClick={this.onCrop.bind(this)}>適用する</button>
              </div>
            </div>
          </div>
        </div>
      </Fragment>
    );
  }
}
